@@ -2,6 +2,7 @@ import re
 import typing
 from collections import Counter, defaultdict
 from dataclasses import dataclass, field
+from enum import Enum, auto
 from html import escape
 from itertools import chain
 from operator import attrgetter
@@ -15,6 +16,12 @@ from four_letter_blocks.grid import Grid
 from four_letter_blocks.block import Block
 
 
+class RotationsDisplay(Enum):
+    OFF = auto()
+    FRONT = auto()
+    BACK = auto()
+
+
 @dataclass
 class Puzzle:
     HINT = 'Clue numbers are shuffled: 1 Across might not be the top left.'
@@ -28,7 +35,8 @@ class Puzzle:
     across_clues: typing.List[Clue] = field(init=False)
     down_clues: typing.List[Clue] = field(init=False)
     use_suits: bool = False
-    use_text: bool = True
+    is_packed: bool = False
+    rotations_display: RotationsDisplay = RotationsDisplay.OFF
 
     @staticmethod
     def parse(source_file: typing.IO) -> 'Puzzle':
@@ -36,11 +44,13 @@ class Puzzle:
         return Puzzle.parse_sections(*sections)
 
     @staticmethod
-    def parse_sections(title: str,
-                       grid_text: str,
-                       clues_text: str,
-                       blocks_text: str,
-                       old_clues: typing.Dict[str, Clue] = None) -> 'Puzzle':
+    def parse_sections(
+            title: str,
+            grid_text: str,
+            clues_text: str,
+            blocks_text: str,
+            old_clues: typing.Dict[str, Clue] = None,
+            old_blocks: typing.List[typing.List[str]] = None) -> 'Puzzle':
         if old_clues is None:
             old_clues = {}
         grid = Grid(grid_text)
@@ -48,7 +58,7 @@ class Puzzle:
         for word, clue in parsed_clues.items():
             old_clues[word] = clue
         all_clues = {}
-        blocks = Block.parse(blocks_text, grid)
+        blocks = Block.parse(blocks_text, grid, old_blocks)
         for block in blocks:
             for square in block.squares:
                 if square.number is None:
@@ -202,7 +212,7 @@ class Puzzle:
             if is_active_row:
                 block.x = x
                 block.y = y
-                block.draw(painter, use_text=self.use_text)
+                block.draw(painter, is_packed=self.is_packed)
             line_height = max(line_height, block.height)
             x += block.width + gap
         if row_index is None:
@@ -361,10 +371,47 @@ class Puzzle:
                            f'{(x2 + 1, y2 + 1)}')
 
     @property
-    def shape_counts(self):
-        return Counter(block.shape
-                       for block in self.blocks
-                       if block.shape is not None)
+    def shape_counts(self) -> typing.Counter[str]:
+        if self.rotations_display == RotationsDisplay.OFF:
+            return Counter(block.shape
+                           for block in self.blocks
+                           if block.shape is not None)
+        counter = Counter()
+        for block in self.blocks:
+            shape = block.shape
+            rotation = block.shape_rotation
+            if self.rotations_display == RotationsDisplay.BACK:
+                rotation = -rotation % 4
+                match shape:
+                    case 'S':
+                        shape = 'Z'
+                    case 'Z':
+                        shape = 'S'
+                    case 'J':
+                        shape = 'L'
+                    case 'L':
+                        shape = 'J'
+            match shape:
+                case None:
+                    continue
+                case 'I' | 'S' | 'Z':
+                    shape = f'{shape}{rotation%2}'
+                case 'J' | 'L' | 'T':
+                    shape = f'{shape}{rotation}'
+            counter[shape] += 1
+        return counter
+
+    @property
+    def flipped_shape_counts(self):
+        if self.rotations_display != RotationsDisplay.OFF:
+            raise RuntimeError(
+                ' flipped_shape_counts is incompatible with rotations display.')
+        shape_counts = self.shape_counts
+        shape_counts['J'], shape_counts['L'] = (shape_counts['L'],
+                                                shape_counts['J'])
+        shape_counts['S'], shape_counts['Z'] = (shape_counts['Z'],
+                                                shape_counts['S'])
+        return shape_counts
 
     @property
     def shape_blocks(self):
@@ -408,7 +455,7 @@ class Puzzle:
 
     def build_clues(self, document: QTextDocument, show_link=True):
         cursor = QTextCursor(document)
-        cursor.movePosition(cursor.End)
+        cursor.movePosition(QTextCursor.End)
 
         font_size = document.defaultFont().pixelSize()
         padding = font_size // 5
@@ -417,7 +464,7 @@ h1 {{text-align: center}}
 hr.footer {{line-height:10px}}
 p.footer {{page-break-after: always}}
 td {{padding: {padding} }}
-td.num {{text-align: right; background: {self.face_colour.name()}}}
+td.num {{text-align: right}}
 a {{color: black}}
 """)
         across_table = build_clue_table(self.across_clues)
@@ -444,7 +491,7 @@ a {{color: black}}
                 <p></p>
             """)
         cursor.insertHtml(html)
-        cursor.movePosition(cursor.End)
+        cursor.movePosition(QTextCursor.End)
 
     def build_hints(self):
         hints = self.HINT
